//
//  MedtechDetectorConstruction.cc
//  medtech
//
//  Created by Baranyai David on 2018. 04. 01..
//

#include "MedtechDetectorConstruction.hh"

MedtechDetectorConstruction::MedtechDetectorConstruction() : G4VUserDetectorConstruction()
{
    
}

MedtechDetectorConstruction::~MedtechDetectorConstruction()
{
    
}

G4VPhysicalVolume* MedtechDetectorConstruction::Construct()
{
    Parameters *parameter = Parameters::getInstance();
    
    //Material Section
    G4NistManager *manager = G4NistManager::Instance();
    G4Material *air = manager -> FindOrBuildMaterial("G4_AIR");
    G4Material *targetMaterial = manager -> FindOrBuildMaterial("G4_Pb");
    //G4Material *collMaterial = manager -> FindOrBuildMaterial("G4_W");
    G4Material *boxMaterial = manager -> FindOrBuildMaterial("G4_W");
    G4Material *tetMaterial = manager -> FindOrBuildMaterial("G4_Fe");
    //End of Material Section
    
    //Physical world
    G4int world_xyz = 1*m; // world is 1m x 1m x 2m for each
    G4Box *solidWorld = new G4Box("World",0.5*world_xyz,0.5*world_xyz,4*m); //define a box for world
    
    G4LogicalVolume *logicWorld = new G4LogicalVolume(solidWorld, air, "World", 0, 0, 0); //define a logical volume for world
    
    G4PVPlacement *physicalWorld = new G4PVPlacement(0, //no rotation
                                                     G4ThreeVector(), //at (0,0,0)
                                                     logicWorld, //it's logical volume
                                                     "World", //it's name
                                                     0, //it's mother volume
                                                     false, //no boolean operations
                                                     0); //no magnetic field
    //End of Physical world
    
    //Target box
    G4Box *solidTarget = new G4Box("Target", 1*m, 1*m, 0.1*cm);
    
    G4LogicalVolume *logicTarget = new G4LogicalVolume(solidTarget, targetMaterial, "Collimator", 0, 0, 0);
    
    G4PVPlacement *physicalTarget = new G4PVPlacement(0, //no rotation
                                                      G4ThreeVector(0, 0, 1.1*m), //at (0,0,0)
                                                      logicTarget, //it's logical volume
                                                      "Target", //it's name
                                                      logicWorld, //it's mother volume
                                                      false, //no boolean operations
                                                      0); //no particular field
    //End of Target box
    
    //Collimator
    G4double box_size = parameter -> GetBoxSize();
    G4Box *solidB1 = new G4Box("B1", 25*cm, 25*cm, 1*cm);
    G4Box *solidB2 = new G4Box("B2", box_size*cm, box_size*cm, 1.5*cm);
    
    G4SubtractionSolid *Box = new G4SubtractionSolid("Box", solidB1, solidB2);
    G4LogicalVolume *logicBox = new G4LogicalVolume(Box, boxMaterial, "Box", 0, 0, 0);
    G4PVPlacement *physicalBox = new G4PVPlacement(0, //no rotation
                                                   G4ThreeVector(0, 0, 10*cm),
                                                   logicBox,
                                                   "Box",
                                                   logicWorld,
                                                   false,
                                                   0);
    //End of Collimator
    
    //Cone
    G4Tet *solidTet = new G4Tet("solidTet", G4ThreeVector(0, 0, 1*cm), G4ThreeVector(-5*cm, -5*cm, 0), G4ThreeVector(5*cm, -5*cm, 0), G4ThreeVector(0, 5*cm, 0));
    G4LogicalVolume *logicTet = new G4LogicalVolume(solidTet, tetMaterial, "Tet", 0, 0, 0);
    G4PVPlacement *physicalTet = new G4PVPlacement(0,
                                                   G4ThreeVector(0,0,3*cm),
                                                   logicTet,
                                                   "Tet",
                                                   logicWorld,
                                                   false,
                                                   0);
    
    return physicalWorld;
}
